<?php

function pubsubhubbub_page(&$page_ar)
{

	global $params,$ut,$config,$html;

	snippet_breadcrumb(0, 'pubsubhubbub');
	
	if (!isset($_GET['id'])) {
		return 'error';
	}

	$id = (int)$_GET['id'];
	$u = $ut->link($id,'prefix:');
	if (!empty($config['canonical'])) {
		if (substr($u,0,strlen($config['site_url'])) == $config['site_url']) {
			$u = $params['canonical'] = $config['canonical'].substr($u,strlen($config['site_url']));
		}
		if (substr($u,0,strlen($config['site_ssl'])) == $config['site_ssl']) {
			$u = $params['canonical'] = $config['canonical'].substr($u,strlen($config['site_ssl']));
		}
	}

	$buff = '';
	if (isset($_POST['mdx'])) {
	    
	    $hub_url = sanitize_str($_POST['hub_url']);
	    $topic_url = sanitize_str($_POST['topic_url']);
	    
$buff .= <<<EOT
		<h1>PubSubHubbub-Publisher</h1>
EOT;
	    // check that a hub url is specified
	    if (!$hub_url) {
$buff .= <<<EOT
			<p>Please specify a hub url.</p>
EOT;
	    }
	    // check that a topic url is specified
	    if (!$topic_url) {
$buff .= <<<EOT
			<p>Please specify a topic url to publish.</p>
EOT;
	    }
	    
	    if ($hub_url && $topic_url) {
		    // $hub_url = "http://pubsubhubbub.appspot.com/publish";
		    $p = new Publisher($hub_url);
		    if ($p->publish_update($topic_url)) {
$buff .= <<<EOT
				<h2>Published successfully</h2>
				<p>
					<i>$topic_url</i> was successfully published to <i>$hub_url</i>
				</p>
				<p><a href='$u' class='onethird-button'>back</a></p>
EOT;
		    } else {
		        $buff .= "<p>ooops...</p>";
		        $buff .= "<pre>".print_r($p->last_response(),false)."</pre>";
		    }
	    }

	} else {
	    
$buff .= <<<EOT
	<h1>PubSubHubbub-Publisher</h1>
	<div class='onethird-setting'>
		<form action='{$_SERVER['REQUEST_URI']}' method='POST'>
			<input type='hidden' name='mdx' value='POST' />
			<input type='hidden' name='xtoken' value='{$params['login_user']['meta']['magic_str']}' />
			<p>hub url<br />
			<input name='hub_url' type='text' value='http://pubsubhubbub.appspot.com/publish' size='50'/></p>
			<p>topic url<br />
			<input name='topic_url' type='text' value='$u' size='50' /></p>
			<p><input name='sub' type='submit' value='Publish' class='onethird-button' /></p>
		</form>
	</div>
EOT;
	}

	return $buff;

}

/**
 * a PHP client library for pubsubhubbub
 * 
 * @link    https://github.com/pubsubhubbub/
 * @author  Josh Fraser | joshfraser.com | josh@eventvue.com
 * @license Apache License 2.0
 */

class Publisher {
    /** @var string */
    protected $hub_url;
    
    /** @var string */
    protected $last_response;
    
    /**
     * create a new Publisher
     * 
     * @param string $hub_url
     */
    public function __construct($hub_url) {
        if (!isset($hub_url))
            throw new \Exception('Please specify a hub url');
        
        if (!preg_match("|^https?://|i",$hub_url)) 
            throw new \Exception('The specified hub url does not appear to be valid: '.$hub_url);
            
        $this->hub_url = $hub_url;
    }
    /**
     * accepts either a single url or an array of urls
     * 
     * @param  string|array $topic_urls
     * @param  callable     $http_function
     * @return mixed
     */
    public function publish_update($topic_urls, $http_function = false) {
        if (!isset($topic_urls))
            throw new \Exception('Please specify a topic url');
        
        // check that we're working with an array
        if (!is_array($topic_urls)) {
            $topic_urls = array($topic_urls);
        }
        
        // set the mode to publish
        $post_string = "hub.mode=publish";
        // loop through each topic url 
        foreach ($topic_urls as $topic_url) {
            // lightweight check that we're actually working w/ a valid url
            if (!preg_match("|^https?://|i",$topic_url)) 
                throw new \Exception('The specified topic url does not appear to be valid: '.$topic_url);
            
            // append the topic url parameters
            $post_string .= "&hub.url=".urlencode($topic_url);
        }
        
        // make the http post request and return true/false
        // easy to over-write to use your own http function
        if ($http_function)
            return $http_function($this->hub_url,$post_string);
        else
            return $this->http_post($this->hub_url,$post_string);
    }
    /**
     * returns any error message from the latest request
     * 
     * @return string
     */
    public function last_response() {
        return $this->last_response;
    }
    
    /**
     * default http function that uses curl to post to the hub endpoint
     * 
     * @param  string $url
     * @param  string $post_string
     * @return bool
     */
    private function http_post($url, $post_string) {
        // add any additional curl options here
        $options = array(CURLOPT_URL => $url,
                         CURLOPT_POST => true,
                         CURLOPT_POSTFIELDS => $post_string,
                         CURLOPT_USERAGENT => "PubSubHubbub-Publisher-PHP/1.0");
    	$ch = curl_init();
    	curl_setopt_array($ch, $options);
        $response = curl_exec($ch);
        $this->last_response = $response;
        $info = curl_getinfo($ch);
        curl_close($ch);
        
        return ($info['http_code'] == 204);
    }
}

?>